import ctypes
from .__library__ import lib, get_error
from .camera_config import *
from .camera import *


class IntegrationTime:
    """
    The integration time as fixed presets
    """

    SHORT = 0
    "250 us"

    MEDIUM = 1
    "500 us"

    LONG = 2
    "1000us"



class ImagingEnvironment:
    """
    The different imaging environments
    """

    INSIDE = 0
    "Inside buildings"

    SUNLIGHT = 1
    "When bright sunlight is present"



class Strategy:
    """
    The optimization of the camera configuration
    """

    BALANCED = 0
    "The balanced configuration"

    SPEED = 1
    "Optimize for moving objects"

    ACCURACY = 2
    "Optimize for high accuracy"



class UserConfig():
    """
    Simple interface for camera configuration generation
    """
    def __del__(self):
        if self._ptr:
            lib.tof_user_config_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, ptr = None):
        """
        Construct new user_config class
        ----------

        Create default user config
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        else:
            self._ptr = lib.tof_user_config_new_default(get_error())


    def toCameraConfig(self, camera):
        """
        Convert to a camera configuration

        Parameters
        ----------
        camera : Camera
            The camera

        Returns
        ----------
        CameraConfig
            The camera configuration
        """
        return CameraConfig(ptr = lib.tof_user_config_to_camera_config(self._ptr, camera._ptr, get_error()))


    def getEnvironment(self):
        """
        Get the camera operating environment

        Returns
        ----------
        ImagingEnvironment
            Imaging environment
        """
        return lib.tof_user_config_get_environment(self._ptr, get_error())


    def setEnvironment(self, imaging_environment):
        """
        Set the camera operating environment

        Parameters
        ----------
        imaging_environment : ImagingEnvironment
            Imaging environment
        """
        return lib.tof_user_config_set_environment(self._ptr, imaging_environment, get_error())


    def getMaxDistance(self):
        """
        Get the maximum distance photons reflect from

        Returns
        ----------
        float
            Maximum distance in meters
        """
        return lib.tof_user_config_get_max_distance(self._ptr, get_error())


    def setMaxDistance(self, max_distance):
        """
        Set the maximum distance photons reflect from

        Parameters
        ----------
        max_distance : float
            Maximum distance in meters
        """
        return lib.tof_user_config_set_max_distance(self._ptr, max_distance, get_error())


    def getFps(self):
        """
        Get the depth frame rate

        Returns
        ----------
        float
            The depth fps
        """
        return lib.tof_user_config_get_fps(self._ptr, get_error())


    def setFps(self, fps):
        """
        Set the depth frame rate

        Parameters
        ----------
        fps : float
            The depth fps
        """
        return lib.tof_user_config_set_fps(self._ptr, fps, get_error())


    def getLensFlare(self):
        """
        Get if lens flare is expected in the scene

        Returns
        ----------
        bool
            if lens flare is present
        """
        return lib.tof_user_config_get_lens_flare(self._ptr, get_error())


    def setLensFlare(self, lens_flare):
        """
        Set if lens flare is expected in the scene

        Parameters
        ----------
        lens_flare : bool
            if lens flare is present
        """
        return lib.tof_user_config_set_lens_flare(self._ptr, lens_flare, get_error())


    def getTranslucent(self):
        """
        Get if there are translucent objects in the scene

        Returns
        ----------
        bool
            If translucent objects
        """
        return lib.tof_user_config_get_translucent(self._ptr, get_error())


    def setTranslucent(self, translucent):
        """
        Set if there are translucent objects in the scene

        Parameters
        ----------
        translucent : bool
            If translucent objects
        """
        return lib.tof_user_config_set_translucent(self._ptr, translucent, get_error())


    def getHdr(self):
        """
        Get high dynamic range enabled. Enable this when pixel saturation occurs.
        This will only happen for retroreflectors or reflective objects close
        (<20cm) to the camera

        Returns
        ----------
        bool
            Enable high dynamic range
        """
        return lib.tof_user_config_get_hdr(self._ptr, get_error())


    def setHdr(self, hdr):
        """
        Set high dynamic range enabled. Enable this when pixel saturation occurs.
        This will only happen for retroreflectors or reflective objects close
        (<20cm) to the camera

        Parameters
        ----------
        hdr : bool
            Enable high dynamic range
        """
        return lib.tof_user_config_set_hdr(self._ptr, hdr, get_error())


    def getIntegrationTime(self):
        """
        Get the integration time

        Returns
        ----------
        IntegrationTime
            The integration time
        """
        return lib.tof_user_config_get_integration_time(self._ptr, get_error())


    def setIntegrationTime(self, int_time):
        """
        Set the integration time

        Parameters
        ----------
        int_time : IntegrationTime
            The integration time
        """
        return lib.tof_user_config_set_integration_time(self._ptr, int_time, get_error())


    def getStrategy(self):
        """
        Get what the configuration is being optimized for

        Returns
        ----------
        Strategy
            The user configuration strategy
        """
        return lib.tof_user_config_get_strategy(self._ptr, get_error())


    def setStrategy(self, strategy):
        """
        Set what the configuration is being optimized for

        Parameters
        ----------
        strategy : Strategy
            The user configuration strategy
        """
        return lib.tof_user_config_set_strategy(self._ptr, strategy, get_error())


    def getChannel(self):
        """
        Get The camera channel, increment to support multi-camera usage

        Returns
        ----------
        int
            The channel number, set to 0, otherwise increment with number of
            cameras.
        """
        return lib.tof_user_config_get_channel(self._ptr, get_error())


    def setChannel(self, channel):
        """
        Set The camera channel, increment to support multi-camera usage

        Parameters
        ----------
        channel : int
            The channel number, set to 0, otherwise increment with number of
            cameras.
        """
        return lib.tof_user_config_set_channel(self._ptr, channel, get_error())


def hdrPossible(config):
    """
    Check whether hdr is possible with current camera config

    Parameters
    ----------
    config : CameraConfig
        The camera config to check

    Returns
    ----------
    bool
        Whether hdr is possible
    """
    return lib.tof_hdr_possible(config._ptr, get_error())


def phaseUnwrapPossible(config):
    """
    Check whether phase unwrap is possible with current camera config

    Parameters
    ----------
    config : CameraConfig
        The camera config to check

    Returns
    ----------
    bool
        Whether phase unwrap is possible
    """
    return lib.tof_phase_unwrap_possible(config._ptr, get_error())


def applyDefaultProcessingConfig(camera):
    """
    Generate processing config based on camera configuration and calibration.

    Parameters
    ----------
    camera : Camera
        A camera
    """
    return lib.tof_apply_default_processing_config(camera._ptr, get_error())


